
#include "perlin.hpp"
#include "../external/perlin/simplexnoise1234.hpp"
#include "../lib3d/vec2.hpp"
#include "../lib3d/vec3.hpp"
#include "../lib3d/vec4.hpp"


#include <cmath>

namespace cpe
{

double perlin::noise(const double x,int octave,double persistency)
{

    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise1(x*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const vec2& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise2(x.x()*frequency,x.y()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const vec3& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise3(x.x()*frequency,x.y()*frequency,x.z()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const vec4& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise4(x.x()*frequency,x.y()*frequency,x.z()*frequency,x.w()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

}
